import {
  NButton,
  NDataTable,
  NFormItem,
  NInput,
  NModal,
  NSelect,
  NSpace,
  NTag,
  useDialog,
  useMessage
} from 'naive-ui'
import { defineComponent, onMounted, reactive, ref } from 'vue'
import { useRoute, useRouter } from 'vue-router'
import { useTabsViewStore } from '@/store/modules/tabsView'
import SaveForm from '@components/save-form'
import Pagination from '@components/pagination'
import deepClone from '@/utils/deep.clone'
import EmailConfigEdit from '@views/message/message-config/email/modal/email-config-edit'
import { getMapValueByKey } from '@/utils/objectUtil'
import { appKey } from '@/utils/constant'
import { sysApplicationPage } from '@views/menu-manage/api'
import { emailConfigPage, emailConfigRemove, emailConfigStatus } from '@views/message/api'

export default defineComponent({
  name: 'email-config',
  setup(props, ctx) {
    const route = useRoute()
    const router = useRouter()
    const dialog = useDialog()
    const message = useMessage()
    const state = reactive({
      loading: false,
      appId: null as any,
      pagination: {
        page: 1,
        rows: 10,
        pageTotal: 0
      },
      searchForm: {
        keyword: null, //关键字
        status: null, // 状态
        appKey: null // 接入应用
      },
      name: null as any,
      dataList: [],
      showEdit: false,
      editMode: 'add',
      editRowData: {} as any,
      appData: [] as any
    })
    const tabsViewStore = useTabsViewStore()
    const gotoBack = () => {
      tabsViewStore.closeCurrentTab(route)
      router.push({
        path: '/message/messageConfig'
      })
    }

    onMounted(async () => {
      // 应用
      {
        state.appData = []
        const { data } = await sysApplicationPage({ page: 1, rows: 999 })
        if (data && data.rows) {
          data.rows.forEach((item: any) => {
            state.appData.push({ label: item.appName, value: item.appKey })
          })
        }
      }

      getList()
    })

    const saveForm = ref()
    const onSearch = () => {
      saveForm.value?.submit()
    }
    const onBtnReset = () => {
      saveForm.value?.reset()
    }

    const onSubmit = () => {
      state.pagination.page = 1
      getList()
    }

    const getList = async () => {
      try {
        state.loading = true
        const { data } = await emailConfigPage({
          ...state.pagination,
          ...state.searchForm
        })
        state.pagination.pageTotal = Number(data.total)
        state.dataList = data.rows || []
      } catch {}
      state.loading = false
    }

    const onChangeStatus = (row: any) => {
      const statusStr = row.status ? '停用' : '启用'
      dialog.warning({
        title: '提示',
        content: `是否${statusStr}？`,
        positiveText: '确定',
        negativeText: '取消',
        onPositiveClick: async () => {
          try {
            await emailConfigStatus({
              id: row.id,
              status: !row.status
            })
            getList()
            message.success(`${statusStr}成功`)
          } catch {}
        }
      })
    }

    const onRmove = (row: any): void => {
      dialog.warning({
        title: '提示',
        content: `删除"${row.name}"，是否继续？`,
        positiveText: '确定',
        negativeText: '取消',
        onPositiveClick: async () => {
          try {
            await emailConfigRemove(row.id)
            getList()
            message.success('删除成功')
          } catch {}
        }
      })
    }

    const columns = (): any => {
      return [
        {
          title: '编号',
          key: 'id'
        },
        {
          title: '平台名称',
          key: 'name'
        },
        {
          title: '接入应用',
          key: 'appKey',
          render(row: any) {
            return <div>{getMapValueByKey(row.appKey, new Map(Object.entries(appKey)))}</div>
          }
        },
        {
          title: '域名',
          key: 'hostUrl'
        },
        {
          title: '端口',
          key: 'smtpPort'
        },
        {
          title: '用户名',
          key: 'account'
        },
        {
          title: '密码',
          key: 'password'
        },
        {
          title: '发送方',
          key: 'from'
        },
        {
          title: '发送名称',
          key: 'fromName'
        },
        {
          title: '状态',
          key: 'status',
          render(row: any) {
            return (
              <NTag type={row.status ? 'primary' : 'default'}>{row.status ? '启用' : '停用'}</NTag>
            )
          }
        },
        {
          title: '操作',
          key: 'operation',
          fixed: 'right',
          render(row: any) {
            return (
              <NSpace>
                <NButton
                  type="primary"
                  size="small"
                  text
                  v-auth="emailConfig/status1790935300606857218"
                  onClick={() => {
                    onChangeStatus(row)
                  }}
                >
                  {row.status ? '停用' : '启用'}
                </NButton>
                <NButton
                  type="primary"
                  size="small"
                  text
                  v-auth="emailConfig/update1790935437341167617"
                  onClick={() => {
                    state.showEdit = true
                    state.editRowData = deepClone(row)
                    state.editMode = 'edit'
                  }}
                >
                  修改
                </NButton>
                <NButton
                  type="primary"
                  size="small"
                  text
                  v-auth="emailConfig/remove1790935226178932738"
                  disabled={!!row.status}
                  onClick={() => {
                    onRmove(row)
                  }}
                >
                  删除
                </NButton>
              </NSpace>
            )
          }
        }
      ]
    }

    return () => (
      <div class="system-menu-container">
        <div class={['section-container']}>
          <div class="system-menu-container">
            <SaveForm
              ref={saveForm}
              model={state.searchForm}
              onSubmit={onSubmit}
              saveKey="email-config"
              onSetModel={(val: any) => (state.searchForm = val)}
            >
              <NFormItem label="关键字" path="keyword">
                <NInput
                  placeholder="编号/平台名称"
                  v-model:value={state.searchForm.keyword}
                  clearable
                />
              </NFormItem>
              <NFormItem label="接入应用" path="appKey">
                <NSelect
                  placeholder="请选择接入应用"
                  v-model:value={state.searchForm.appKey}
                  options={state.appData}
                  clearable
                />
              </NFormItem>
              <NFormItem label="状态" path="status">
                <NSelect
                  v-model:value={state.searchForm.status}
                  clearable
                  options={
                    [
                      {
                        label: '启用',
                        value: 1
                      },
                      {
                        label: '停用',
                        value: 0
                      }
                    ] as any
                  }
                  placeholder="全部状态"
                />
              </NFormItem>
              <NFormItem>
                <NSpace>
                  <NButton type="primary" onClick={onSearch}>
                    搜索
                  </NButton>
                  <NButton type="default" onClick={onBtnReset}>
                    重置
                  </NButton>
                </NSpace>
              </NFormItem>
            </SaveForm>

            <NSpace style={{ paddingBottom: '12px' }}>
              <NButton
                type="primary"
                v-auth="emailConfig/save1790935373034098690"
                onClick={() => {
                  state.showEdit = true
                  state.editMode = 'add'
                }}
              >
                新增邮件配置
              </NButton>
            </NSpace>

            <div class={['section-container']}>
              <NDataTable
                loading={state.loading}
                columns={columns()}
                data={state.dataList}
                rowKey={(row: any) => row.id}
                scrollX={'1400'}
              ></NDataTable>

              <Pagination
                v-model:page={state.pagination.page}
                v-model:pageSize={state.pagination.rows}
                v-model:pageTotal={state.pagination.pageTotal}
                onList={getList}
                sync
                saveKey="email-config"
              ></Pagination>
            </div>
            <NModal
              blockScroll={true}
              v-model:show={state.showEdit}
              preset="dialog"
              showIcon={false}
              title={(state.editMode == 'add' ? '新增' : '编辑') + '邮件配置'}
              style={{ width: '700px' }}
            >
              <EmailConfigEdit
                editMode={state.editMode}
                rowData={state.editRowData}
                onClose={() => (state.showEdit = false)}
                onGetList={() => {
                  state.pagination.page = 1
                  getList()
                }}
              />
            </NModal>
          </div>
        </div>
      </div>
    )
  }
})
